
#import "AppController.h"
#import "RBSplitViewFixer.h"

static NSUInteger fileCounter = 0;
static NSUInteger correctCounter = 0;
static NSUInteger wrongCounter = 0;

// This class illustrates how to sweep a folder and its subfolders, using the
//	RBSplitViewFixer class, looking for text files and trying to identify their
//	encodings.

@implementation AppController

- (RBSplitViewFixer*) sweeper {
	static RBSplitViewFixer* sweeper = nil;
	if (!sweeper) {
		// Get a RBSplitViewFixer instance, set ourselves as its delegate, don't follow links.
		sweeper = [[RBSplitViewFixer alloc] init];
		[sweeper setDelegate:self];
		[sweeper setFollowsLinks:NO];
	}
	return sweeper;
}

- (void)updateDisplay {
	[textview scrollRangeToVisible:NSMakeRange([[textview textStorage] length],0)];
	[scrollview display];
}

// This NSLog-like method logs to our main window.
- (void)log:(NSString*)format,... {
	va_list va;
	va_start(va,format);
	NSString* str = [[NSString alloc] initWithFormat:format arguments:va];
	va_end(va);
	NSAttributedString* astr = [[NSAttributedString alloc] initWithString:str];
	[str release];
	NSTextStorage* stor = [textview textStorage];
	[stor appendAttributedString:astr];
	[astr release];
	static NSAttributedString* newln = nil;
	if (!newln) {
		newln = [[NSAttributedString alloc] initWithString:@"\n"];
	}
	[stor appendAttributedString:newln];
	// The run loop doesn't get activity while sweeping a folder, so we have to update the text view
	// manually every 0.3 seconds.
	static NSTimeInterval last = 0.0;
	NSTimeInterval now = [NSDate timeIntervalSinceReferenceDate];
	if (now>(last+0.3)) {
		last = now;
		[self updateDisplay];
	}
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender {
	return YES;
}

- (BOOL)application:(NSApplication *)sender openFile:(NSString *)filename {
	fileCounter = 0;
	// Note current date/time so we can work out how long the folder-sweep took.
	NSDate* start = [NSDate date];
	
	[self log:@"\nChecking \"%@\"",filename];
	// We set up a pointer to an NSError, so we can get information on any errors which may occur.
	// You can also just pass NULL for the error: argument if you're not interested in errors.
	NSError *error = nil; 
	BOOL result = [[self sweeper] sweepFolder:filename gettingInfo:0 error:&error]; // pass in 0 for gettingInfo as we don't need file metadata
	if (error) {
		[self log:@"An error occurred: %@", error];
	}
	if (result) {
		// Work out how long we took, and output the results of the sweep.
		NSTimeInterval delta = -[start timeIntervalSinceNow];
		[self log:@"Checked folder \"%@\" in %g seconds, %d files checked.\n", 
		 filename, delta, fileCounter];
		[self updateDisplay];
	}
	return YES;
}

- (void)application:(NSApplication *)sender openFiles:(NSArray *)filenames {
	for (NSString* file in filenames) {
		[self application:sender openFile:file];
	}
}

- (BOOL)sweeper:(RBSplitViewFixer*)sweeper shouldProcessObject:(FSRef*)aFileRef named:(NSString*)aFileName 
					hasInfo:(FSCatalogInfo*)aFileInfo isFolder:(BOOL)isFolder {
	// This method, which must be implemented, is called to determine whether to further process a file or folder.
	// Processing a folder means recursively sweeping its contents, and processing a file means that our 
	//	processFile:named:hasInfo:contents: method will be called for that file.
	[self log:@"Checking \"%@\"%@",aFileName,isFolder?@" and its contents":@""];
	if (!isFolder) {
		++fileCounter;
	}
	return YES;
}

- (void)sweeper:(RBSplitViewFixer*)sweeper processFile:(FSRef*)aFileRef named:(NSString*)aFileName 
			hasInfo:(FSCatalogInfo*)aFileInfo contents:(UInt8*)bytes length:(UInt64)length {
	// This is an optional delegate method for the RBSplitViewFixer's delegate.
	// If implemented, it will be called with the memory-mapped contents of 
	//	each file to which our shouldProcessObject:named:hasInfo:isFolder: 
	//	method returned YES.

	if (!bytes) {
		return;
	}
	correctCounter = 0;
	wrongCounter = 0;
	while (length>0) {
		UInt8* rptr = memchr(bytes, 'R', length);
		if (rptr) {
			UInt64 delta = (++rptr)-bytes;
			length -= delta;
			bytes = rptr;
			if (length>=10) {
				if (memcmp(bytes, "BSplitView", 10)==0) {
					correctCounter++;
					length -= 10;
					bytes += 10;
				} else if (memcmp(bytes, "BsplitView", 10)==0) {
					bytes[1] = 'S';
					wrongCounter++;
					length -= 10;
					bytes += 10;
				}
			} else {
				break;
			}
		} else {
			break;
		}
	}
	if (correctCounter||wrongCounter) {
		// This format string is obfuscated to avoid breaking the source AND object code just in case
		// you stupidly decide to use the project folder as a test folder (and yes, I did it at least twice!)
		[self log:@"\t%d instances of RB%cplitView, %d of RB%cplitView (fixed)",correctCounter,'S',wrongCounter,'s'];
	}
}


@end
